// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2009-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Mono.Unix;

namespace Smdn.IO.FileSystems.Unix {
  internal class UnixFileSystem : IFileSystem {
    public UnixFileSystem()
    {
    }

    public Directory GetDirectory(string directory)
    {
      return new UnixDirectory(new UnixDirectoryInfo(directory));
    }

    public Drive GetDrive(string drive)
    {
      return new UnixDrive(new UnixDriveInfo(drive));
    }

    public IEnumerable<Drive> GetDrives(Predicate<Drive> match)
    {
      foreach (var drive in UnixDrive.GetDrives()) {
        if (match == null || match(drive))
          yield return drive;
      }
    }

    public bool DeviceEquals(DirectoryEntry x, DirectoryEntry y)
    {
      var entryX = (x as IUnixFileSystemEntry);
      var entryY = (y as IUnixFileSystemEntry);

      if (entryX == null || entryY == null)
        return false;
      else
        return entryX.Info.Device == entryY.Info.Device;
    }
  }
}
