//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Text.RegularExpressions;

using Smdn.IO.FileSystems;

namespace Smdn.Applications.BittenBaum.Analysis {
  public class AnalysisContext : TaskContext, ITaskContext {
    public AnalysisContext(Directory analysisDirectory)
    {
      this.parameter = new AnalysisParameter();
      this.parameter.AnalysisDirectory = analysisDirectory;
    }

#region parameters
    private readonly AnalysisParameter parameter;

    public AnalysisParameter Parameter {
      get { return parameter; }
    }
    
    public class AnalysisParameter : TaskParameter {
      public Directory AnalysisDirectory = null;
      public Predicate<DirectoryEntry> ScanPredicate = null;
    }
#endregion

#region status
    private readonly AnalysisStatus status = new AnalysisStatus();

    public AnalysisStatus Status {
      get { return status; }
    }

    public class AnalysisStatus : TaskStatus {
      public float AnalyzedPercentage = 0.0f;

      public override int Total {
        get { return -1; }
      }

      public override int Processed {
        get { throw new NotSupportedException(); }
      }

      public override float Percentage {
        get { return AnalyzedPercentage; }
      }
    }
#endregion

#region results
    private readonly AnalysisResult result = new AnalysisResult();

    public AnalysisResult Result {
      get { return result; }
    }

    public class AnalysisResult : TaskResult {
      public DirectoryTree DirectoryTree = null;
    }
#endregion

    #region ITaskContext

    TaskContext.TaskParameter ITaskContext.Parameter {
      get { return parameter; }
    }

    TaskContext.TaskResult ITaskContext.Result {
      get { return result; }
    }

    TaskContext.TaskStatus ITaskContext.Status {
      get { return status; }
    }

    #endregion
  }
}
