//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Text.RegularExpressions;

using Smdn.IO.FileSystems;

namespace Smdn.Applications.BittenBaum.Analysis {
  public class Analyzer {
    private readonly AnalysisContext context;

    public Analyzer(AnalysisContext context)
    {
      if (context == null)
        throw new ArgumentNullException("context");

      this.context = context;
    }

    public void Analyze()
    {
      // 対象ディレクトリを走査
      context.Status.StatusText = "ディレクトリを解析中";

      var directoryTree = context.Parameter.AnalysisDirectory as DirectoryTree;

      if (directoryTree == null) {
        context.Result.DirectoryTree = DirectoryTree.Create(context.Parameter.AnalysisDirectory,
                                                            context.Parameter.ScanPredicate,
                                                            delegate(Directory directory, float percentage) {
          context.Status.StatusText = string.Concat("scanning ", directory.FullName);
          context.Status.AnalyzedPercentage = percentage;
        });
      }
      else {
        directoryTree.Refresh(context.Parameter.ScanPredicate,
                              delegate(Directory directory, float percentage) {
          context.Status.StatusText = string.Concat("scanning ", directory.FullName);
          context.Status.AnalyzedPercentage = percentage;
        });

        context.Result.DirectoryTree = directoryTree;
      }

      context.Status.StatusText = "ディレクトリの解析が完了しました";
    }
  }
}