//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Text;

using Smdn.IO.FileSystems;

namespace Smdn.Applications.BittenBaum.Formatting {
  using Analysis;
  using Graphing;

  using IO = System.IO;

  partial class Formatter {
    /// <summary>
    /// CSV用のFormatter
    /// </summary>
    private class CsvFormatter {
      private readonly FormattingContext.FormattingParameter parameter;

      public CsvFormatter(FormattingContext.FormattingParameter parameter)
      {
        this.parameter = parameter;
      }

      public void Format(FormattingContext.FormattingStatus status)
      {
        var writer = new IO.StreamWriter(parameter.OutputStream, Encoding.Default);

        status.TotalEntryCount = parameter.FormattingElements.Count;
        status.FormattedEntryCount = 0;

        writer.WriteLine("種類,名前,容量[%],容量[Bytes],フルパス");

        foreach (var element in parameter.FormattingElements) {
          var entry = element.Entry;

          if (!entry.Exists)
            continue; // ignore deleted entry

          string type;
          long size;

          if (entry is File) {
            type = "f";
            size = entry.Length;
          }
          else if (entry is Directory) {
            type = "d";
            size = (entry as DirectoryTree).GetTotalFileSize();
          }
          else {
            continue;
          }

          // Type, Name, Size(%), Size(Bytes), FullName
          writer.WriteLine("{0},\"{1}\",{2:P2},{3},\"{4}\"",
                           type,
                           entry.Name,
                           element.Range,
                           size,
                           entry.FullName);

          status.FormattedEntryCount++;
        }

        status.FormattedEntryCount = status.TotalEntryCount;

        writer.Flush();

        if (parameter.CloseStreamAfterFormatting)
          writer.Close();
      }
    }
  }
}