//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Collections.Generic;
using System.IO;

namespace Smdn.Applications.BittenBaum.Formatting {
  using Graphing;

  public enum FormatType {
    Xml,
    Csv,
  }

  public partial class Formatter {
    private readonly FormattingContext context;

    public Formatter(FormattingContext context)
    {
      if (context == null)
        throw new ArgumentNullException("context");

      this.context = context;
    }

    public void Format()
    {
      context.Status.StatusText = "整形中";

      if (context.Parameter.Comparison != null) {
        var sorted = new List<IGraphElement>(context.Parameter.FormattingElements);

        sorted.Sort(context.Parameter.Comparison);

        context.Parameter.FormattingElements = sorted;
      }

      switch (this.context.Parameter.Type) {
        case FormatType.Xml:
          (new XmlFormatter(context.Parameter)).Format(context.Status);
          break;

        case FormatType.Csv:
          (new CsvFormatter(context.Parameter)).Format(context.Status);
          break;

        default:
          throw new NotImplementedException(string.Format("{0}は整形出来ません", context.Parameter.Type));
      }

      context.Status.StatusText = "整形が完了しました";
    }
  }
}
