//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Collections.Generic;
using System.Text;

using Smdn.IO.FileSystems;

namespace Smdn.Applications.BittenBaum.Graphing.Disc {
  public class Pie : IGraphElement {
    internal static readonly Pie FreeSpace = new FreeSpacePie();

    private class FreeSpacePie : Pie {
      public override DirectoryEntry Entry {
        get { throw new NotSupportedException(); }
      }

      public override TreePie Parent {
        get { throw new NotSupportedException(); }
      }

      public override bool IsFreeSpace {
        get { return true; }
      }

      public override int Depth {
        get { throw new NotSupportedException(); }
      }

      public override double Range {
        get { throw new NotSupportedException(); }
        internal set { throw new NotSupportedException(); }
      }

      public override double Argument {
        get { throw new NotSupportedException(); }
        internal set { throw new NotSupportedException(); }
      }

      public FreeSpacePie()
        : base(null, null)
      {
      }
    }


    public virtual DirectoryEntry Entry {
      get; private set;
    }

    public virtual TreePie Parent {
      get; private set;
    }

    public virtual bool IsFreeSpace {
      get { return false; }
    }

    ITreeGraphElement IGraphElement.Parent {
      get { return Parent; }
    }

    public virtual int Depth {
      get {
        if (Parent == null)
          return 0;
        else
          return Parent.Depth + 1;
      }
    }

    public Pie(DirectoryEntry entry, TreePie parent)
    {
      this.Entry = entry;
      this.Parent = parent;
    }

    public Pie GetParent(int depth)
    {
      var d = Depth;

      for (Pie p = this; p != null; p = p.Parent) {
        if (d-- == depth)
          return p;
      }

      return null;
    }

    public virtual double Range {
      get; internal set;
    }

    public virtual double Argument {
      get; internal set;
    }
  }
}
